# -*- coding: utf8 -*-

import hashlib
import random
import socket
import urllib
import urllib2
import cookielib

from smtplib import SMTPException 

from django.conf import settings
from django.db import models, IntegrityError
from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext as _
from django.views.generic.base import TemplateView
from django.views.generic.edit import CreateView

from .forms import SubscribeForm
from .models import Subscriber


class SubscribeView(CreateView):
    form_class = SubscribeForm
    model = Subscriber

    success_url = 'thanks'
subscribe = SubscribeView.as_view()


class ThanksView(TemplateView):
    template_name = 'newsletter/thanks.html'
thanks = ThanksView.as_view()


class ConfirmationView(TemplateView):
    template_name = 'newsletter/done.html'

    def get_context_data(self, token, **kwargs):
        context = super(ConfirmationView, self).get_context_data(**kwargs)
        subscriber = get_object_or_404(Subscriber, password=token, is_registered=None)
        subscriber.is_validated = True
        subscriber.save()
        subscriber.subscribe_in_mailman()
        return context
confirmation = ConfirmationView.as_view()


def unsubscription(request) :
    UNSUBSCRIPTION_MESSAGE = _("Enter your email for unsunscribe to the newsletter.")
    UNSUBSCRIPTION_CONFIRMATION_MESSAGE = _("You're now unsubscribed")

    if request.method == 'GET' :
        form = SubscriptionForm()
        return render(request, 'unsubscription_form.html', {'form' : form, 'message' : UNSUBSCRIPTION_MESSAGE})
    elif request.method == 'POST' :
        form = SubscriptionForm(request.POST)
        if form.is_valid() :
            cd = form.cleaned_data
            try :
                subscriber = Subscriber.objects.get(email = cd['email'])
                
                cookie_jar = cookielib.CookieJar()
                url_opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(cookie_jar))

                login_params = urllib.urlencode({'email' : subscriber.email, 'password' : subscriber.password})
                login_request = urllib2.Request(settings.NEWSLETTER_UNSUBSCRIPTION_URL, login_params)
                url = url_opener.open(login_request)
                
                unsubscribe_action_url = ("%s/%s" % (settings.NEWSLETTER_UNSUBSCRIPTION_URL, subscriber.email))
                unsubscribe_params = urllib.urlencode({'unsub' : 'Résilier', 'unsubconfirm' : '1'})
                unsubscribe_request = urllib2.Request(unsubscribe_action_url, unsubscribe_params)
                url = url_opener.open(unsubscribe_request)

                url.close()

                subscriber.delete()
                return render(request, 'unsubscription_form.html', {'message' : UNSUBSCRIPTION_CONFIRMATION_MESSAGE})
            except Subscriber.DoesNotExist :
                error_message = _("The entered email address does not exist. Please try another one.")
                return render(request, 'unsubscription_form.html', {'form' : form, 'message' : error_message})
